/*
**       +----------------------------------------------------------------+
**       | read_gpsdata.c check NW Interfaces to startup LoRa GW software |
**       +----------------------------------------------------------------*
**
**       Startup Program to check Network Connectivity and Display IP@
**       added feature to query GPS-Daemon to replace Coordinates in APRS.conf
**       before starting up iot4p-GW-Software
**
**       additional interfaces can be added thru 'ret = show_IP( "wlan0" );'
**       interface names can be retrieved by 'ifconfig (1)'
**
**       needs additional libraries libgps, wiringPI and oled96
**       complie:  make     (using supplied Makefile)
**              ( cc show_startup.o get_gpsdata.o -loled96 -lm -lpthread
**                      -lwiringPi -lgps -lm -o show_startup )
**
**	 needs 'gpsd' running on the system with valid GPS-source (either USB/
**		Serial/BT or Network )
**		add devices to gpsd: sudo gpsdctl add tcp://192.168.2.107:2947 
**
**       Author: OE1ACM / Bernd Gasser   lora-aprs.at   2020/05/03
**
*/


#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>


#include <stddef.h>
#include <errno.h>
#include <math.h>

#include <gps.h>


#define	GPS_ERR		-1
#define	GPSD_ERR	-2
#define GPS_NODATA	0
#define	GPS_VALID 	2


double  conv_coord( double x );

struct gps_data_t GPS_data;

extern	print_OLED( int , char * );
extern  int	verbose;


int build_APRSconf(void) {

char	latitud[64];
char	longitud[64];
char	outstring[512];

int	fd_out=0;
int	wr=0;
int	try_gps = 10;  // retry 10 Times

int	ret = 0;


	memset(latitud,0,sizeof(latitud) );
	memset(longitud,0,sizeof(longitud) );
	memset(outstring,0,sizeof(outstring) );







    while (  try_gps-- ) {


	ret = query_gpsd( 300  );

	if ( ret == GPSD_ERR ) {
		// no gpsd running - give up 
		break;
	}

	if ( ret >= GPS_VALID ) {
        	sprintf(latitud,"%3.2f%c",  
			 conv_coord( GPS_data.fix.latitude ),  GPS_data.fix.latitude > 0 ? 'N':'S' );
        	sprintf(longitud,"%08.2f%c", 
			conv_coord(GPS_data.fix.longitude), GPS_data.fix.longitude > 0  ? 'E':'W' );

		sprintf(outstring,
			"s/LATITUDE:________/LATITUDE:%s/g\ns/LONGITUDE:_________/LONGITUDE:%s/g\n", 
			latitud,longitud);


		if( verbose ) printf("%s\n", outstring);

		unlink("sedfile");

		fd_out=open("sedfile", O_WRONLY|O_CREAT,0777);
		wr=write(fd_out,outstring,strlen(outstring) );
		close(fd_out);

		break;
	}

	sleep(1);
    }

  if ( ret == GPS_VALID ) {
	// we got valid GPS Position and can sed APRS.conf with new GPS Data

	if( verbose ) printf("writing APRS.conf\n");
	system("sed -f sedfile APRS.conf.GPS_in > APRS.conf.GPS");
    	return EXIT_SUCCESS;
   } else {

	if( verbose ) printf("failure - no new APRS.conf built, using old one\n");
	return GPS_ERR;
   }
}


double	conv_coord( double x ) {

int	xint=0;
double	min=0.0;
double	result=0.0;

	xint = (int)(x);
	min  = ( x - xint ) * 60;
	result = xint*100 + min ;

	return result;
}






int	query_gpsd( int loopcnt )
{
        int ret;
	int	i;
	char	mstring[64];


        ret = gps_open("localhost", "2947", &GPS_data);

	if ( ret )  {
		if( verbose ) printf("ERROR %d contacting gpsd!\n", ret );
		return GPSD_ERR;
		//exit(1);
	}


        (void)gps_stream(&GPS_data, WATCH_ENABLE | WATCH_JSON, NULL);

        for (i=0 ; i <= loopcnt ; i++ ) {

                /* Put this in a loop with a call to a high resolution sleep () in it. */
                if (gps_waiting(&GPS_data, 5000 )) {
			
                        errno = 0;
                        if (gps_read(&GPS_data) == -1) {
                                if( verbose ) printf("ERROR reading GPS_data!\n");
                                //exit(1);
				return GPS_ERR;

                        } else {
                                if (GPS_data.set && GPS_data.status && !isnan( GPS_data.fix.latitude ) ) {
				   if(verbose) {
                                        printf("\n--- GPS --- loop:%d\n", i );
                                        printf("GPS_data.online:           %10.0f\n", GPS_data.online);
                                        printf("GPS_data.status:           %d\n", GPS_data.status);
                                        printf("GPS_data.satellites_used:  %d\n", GPS_data.satellites_used);
                                        printf("GPS_data.fix.mode:         %d\n", GPS_data.fix.mode);
                                        printf("GPS_data.fix.time:         %10.0f\n", GPS_data.fix.time);
                                        printf("GPS_data.fix.latitude:     %f\n", GPS_data.fix.latitude);
                                        printf("GPS_data.fix.longitude:    %f\n", GPS_data.fix.longitude);
                                        printf("GPS_data.fix.altitude:     %f\n", GPS_data.fix.altitude);
                                        printf("GPS_data.fix.speed:        %f\n", GPS_data.fix.speed);
                                        printf("GPS_data.fix.track:        %f\n", GPS_data.fix.track);
                                        printf("GPS_data.dop.pdop:         %f\n", GPS_data.dop.pdop);
				   }	
					// print position to OLED
					sprintf(mstring,"Found GPS Position"); 			print_OLED( 3, mstring );
					sprintf(mstring,"Lat: %7.5f", GPS_data.fix.latitude ); 	print_OLED( 4, mstring );
					sprintf(mstring,"Lon: %7.5f", GPS_data.fix.longitude );	print_OLED( 5, mstring );
					

					return GPS_VALID;
                                }
                        }
                }
		// if( verbose ) printf("errno from gps_waiting: %d   loop: %d GPS_data.set : %d \n",errno, i , GPS_data.set );
        }
	//return GPS_data.status ;
	(void) gps_stream(&GPS_data, WATCH_DISABLE, NULL);
	(void) gps_close (&GPS_data);

	return GPS_NODATA;
}
